==> auto_animated-1.4.0-dev.1.tar.gz: lib/src/on_visibility_change.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_widgets/flutter_widgets.dart';

typedef AutoAnimatedBuilder = Widget Function(
  BuildContext context,
  Animation<double> animation,
);

class AnimateOnVisibilityChange extends StatefulWidget {
  const AnimateOnVisibilityChange({
    @required Key key,
    @required this.builder,
    this.duration = const Duration(milliseconds: 300),
    this.delay = Duration.zero,
  }) : super(key: key);

  final AutoAnimatedBuilder builder;
  final Duration duration, delay;

  @override
  _AnimateOnVisibilityChangeState createState() =>
      _AnimateOnVisibilityChangeState();
}

class _AnimateOnVisibilityChangeState extends State<AnimateOnVisibilityChange>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  _VisibilityStackProvider _wrapper;

  @override
  void initState() {
    _wrapper = _VisibilityStackProvider.of(context);
    _controller = AnimationController(
      duration: widget.duration,
      vsync: this,
    );
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => VisibilityDetector(
        key: widget.key,
        child: widget.builder(
          context,
          _controller.view,
        ),
        onVisibilityChanged: _visibilityChanged,
      );

  void _visibilityChanged(VisibilityInfo info) {
    if (_controller.isAnimating) {
      return;
    }
    if (info.visibleFraction > 0.025 && !_controller.isCompleted) {
      Future.delayed(widget.delay, () {
        if (!mounted) {
          return;
        }
        if (_wrapper != null) {
          _wrapper.stack.add(() {
            _controller.forward();
          });
        } else {
          _controller.forward();
        }
      });
    } else if (info.visibleFraction <= 0.025 && mounted) {
      _controller.reverse();
    }
  }
}

class AnimateOnVisibilityWrapper extends StatefulWidget {
  const AnimateOnVisibilityWrapper({
    @required this.child,
    this.showItemInterval = const Duration(milliseconds: 150),
    Key key,
  }) : super(key: key);

  final Widget child;
  final Duration showItemInterval;

  @override
  _AnimateOnVisibilityWrapperState createState() =>
      _AnimateOnVisibilityWrapperState();
}

class _AnimateOnVisibilityWrapperState
    extends State<AnimateOnVisibilityWrapper> {
  _VisibilityStack _stack;
  double _lastScrollExtend = 0;

  @override
  void initState() {
    _stack = _VisibilityStack(widget.showItemInterval);
    super.initState();
  }

  @override
  void dispose() {
    _stack.dispose();
    super.dispose();
  }

  @override
  void didUpdateWidget(AnimateOnVisibilityWrapper oldWidget) {
    if (oldWidget.showItemInterval != widget.showItemInterval) {
      _stack.showItemInterval = widget.showItemInterval;
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) => _VisibilityStackProvider(
        stack: _stack,
        child: NotificationListener<ScrollNotification>(
          child: widget.child,
          onNotification: _onScroll,
        ),
      );

  bool _onScroll(ScrollNotification scrollInfo) {
    if (scrollInfo.metrics.pixels > _lastScrollExtend + 2.5) {
      // to end
      _stack.direction = AnimationDirection.toEnd;
    } else if (scrollInfo.metrics.pixels < _lastScrollExtend - 2.5) {
      // to start
      _stack.direction = AnimationDirection.toStart;
    }
    _lastScrollExtend = scrollInfo.metrics.pixels;
    return true;
  }
}

class _VisibilityStackProvider extends InheritedWidget {
  _VisibilityStackProvider({
    @required Widget child,
    @required this.stack,
    Key key,
  })  : assert(child != null),
        super(key: key, child: child);

  final _VisibilityStack stack;

  static _VisibilityStackProvider of(BuildContext context) => context
      .ancestorInheritedElementForWidgetOfExactType(_VisibilityStackProvider)
      ?.widget;

  @override
  bool updateShouldNotify(_VisibilityStackProvider oldWidget) =>
      oldWidget.stack != stack;
}

class _VisibilityStack {
  _VisibilityStack(this.showItemInterval);

  Duration showItemInterval;
  AnimationDirection direction = AnimationDirection.toEnd;
  bool _isAnimated = false;

  final List<Function> _stack = [];

  void add(VoidCallback callback) {
    _stack.add(callback);
    _animate();
  }

  void _show() {
    if (direction == AnimationDirection.toEnd) {
      _stack
        ..first.call()
        ..removeAt(0);
    } else {
      _stack
        ..last.call()
        ..removeLast();
    }
  }

  void _animate() {
    if (_isAnimated) {
      return;
    }
    _isAnimated = true;

    Future.delayed(showItemInterval, () {
      if (_stack.isNotEmpty) {
        _show();
        _isAnimated = false;
        _animate();
      } else {
        _isAnimated = false;
      }
    });
  }

  void dispose() {}

  @override
  bool operator ==(Object o) =>
      o is _VisibilityStack && showItemInterval == o.showItemInterval;

  @override
  int get hashCode => showItemInterval.hashCode;
}

enum AnimationDirection {
  toStart,
  toEnd,
}

==> auto_animated-2.0.0-dev.4.tar.gz: lib/src/on_visibility_change.dart <==
import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/widgets.dart';
import 'package:flutter_widgets/flutter_widgets.dart';

import 'animator/stack.dart';

typedef AutoAnimatedBuilder = Widget Function(
  BuildContext context,
  Animation<double> animation,
);

class AnimateOnVisibilityChange extends StatefulWidget {
  const AnimateOnVisibilityChange({
    @required Key key,
    @required this.builder,
    this.delay = Duration.zero,
    this.duration = const Duration(milliseconds: 250),
    this.reAnimateOnVisibility = false,
  })  : assert(delay != null),
        assert(duration != null),
        super(key: key);

  final AutoAnimatedBuilder builder;
  final Duration duration, delay;

  /// Hide the element when it approaches the
  /// frame of the screen so that in the future,
  /// when it falls into the visibility range - reproduce animation again
  final bool reAnimateOnVisibility;

  @override
  _AnimateOnVisibilityChangeState createState() =>
      _AnimateOnVisibilityChangeState();
}

class _AnimateOnVisibilityChangeState extends State<AnimateOnVisibilityChange>
    with SingleTickerProviderStateMixin {
  AnimationController _controller;
  _VisibilityStackProvider _wrapper;

  @override
  void initState() {
    _wrapper = _VisibilityStackProvider.of(context);
    final itemAlreadyShowed = _wrapper.stack.isAlreadyAnimated(widget.key) &&
        !widget.reAnimateOnVisibility;

    _controller = AnimationController(
      value: itemAlreadyShowed ? 1 : 0,
      duration: widget.duration,
      vsync: this,
    );
    super.initState();
  }

  @override
  void dispose() {
    _controller.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) => VisibilityDetector(
        key: widget.key,
        child: widget.builder(
          context,
          _controller.view,
        ),
        onVisibilityChanged: _visibilityChanged,
      );

  void _visibilityChanged(VisibilityInfo info) {
    if (info.visibleFraction > 0.025 && !_controller.isCompleted) {
      Future.delayed(widget.delay, () {
        if (_wrapper != null) {
          _wrapper.stack.add(widget.key, () {
            if (mounted) {
              _controller.forward();
            }
          });
        } else {
          if (mounted) {
            _controller.forward();
          }
        }
      });
    } else if (info.visibleFraction <= 0.025 &&
        mounted &&
        widget.reAnimateOnVisibility &&
        !info.visibleBounds.isEmpty) {
      _controller.reverse();
    }
  }
}

class AnimateOnVisibilityWrapper extends StatefulWidget {
  const AnimateOnVisibilityWrapper({
    @required this.child,
    this.delay = Duration.zero,
    this.showItemInterval = const Duration(milliseconds: 150),
    this.controller,
    Key key,
  })  : assert(delay != null),
        assert(showItemInterval != null),
        super(key: key);

  final ScrollController controller;
  final Widget child;
  final Duration delay, showItemInterval;

  @override
  _AnimateOnVisibilityWrapperState createState() =>
      _AnimateOnVisibilityWrapperState();
}

class _AnimateOnVisibilityWrapperState
    extends State<AnimateOnVisibilityWrapper> {
  VisibilityStack _stack;
  double _lastScrollExtend = 0;

  @override
  void initState() {
    _stack = VisibilityStack(
      delay: widget.delay,
      showItemInterval: widget.showItemInterval,
    );
    if (widget.controller != null && widget.controller.hasClients) {
      widget.controller.addListener(_handleScrollController);
    }
    super.initState();
  }

  @override
  void dispose() {
    _stack.dispose();
    widget.controller?.removeListener(_handleScrollController);
    super.dispose();
  }

  @override
  void didUpdateWidget(AnimateOnVisibilityWrapper oldWidget) {
    if (oldWidget.showItemInterval != widget.showItemInterval) {
      _stack.showItemInterval = widget.showItemInterval;
    }
    super.didUpdateWidget(oldWidget);
  }

  @override
  Widget build(BuildContext context) => _VisibilityStackProvider(
        stack: _stack,
        child: NotificationListener<ScrollNotification>(
          child: widget.child,
          onNotification: _handleScrollNotifications,
        ),
      );

  void _handleScrollController() {
    _setDirection(widget.controller.offset);
  }

  bool _handleScrollNotifications(ScrollNotification scrollInfo) {
    _setDirection(scrollInfo.metrics.pixels);
    return true;
  }

  void _setDirection(double offset) {
    if (offset > _lastScrollExtend + 2.5) {
      // to end
      _stack.direction = AnimationDirection.toEnd;
    } else if (offset < _lastScrollExtend - 2.5) {
      // to start
      _stack.direction = AnimationDirection.toStart;
    }
    _lastScrollExtend = offset;
  }
}

class _VisibilityStackProvider extends InheritedWidget {
  _VisibilityStackProvider({
    @required Widget child,
    @required this.stack,
    Key key,
  })  : assert(child != null),
        super(key: key, child: child);

  final VisibilityStack stack;

  static _VisibilityStackProvider of(BuildContext context) => context
      .getElementForInheritedWidgetOfExactType<_VisibilityStackProvider>()
      ?.widget;

  @override
  bool updateShouldNotify(_VisibilityStackProvider oldWidget) =>
      oldWidget.stack != stack;
}
